/*
		This shader is used for rendering the input/coarse-mesh
		
		Matthias Hollnder 2008/2009
*/

/// --------------------------------------------------
// shader definition
// --------------------------------------------------
#version 120

// --------------------------------------------------
// Attributes
// --------------------------------------------------
attribute vec4 			vWeight;						// weights for the bones
attribute vec4 			vIndex;							// indices of bones
attribute float 		fNumBones;						// number of bones that are connected to this vertex

// --------------------------------------------------
// Uniform variables:
// --------------------------------------------------
uniform mat4 			MatBones[30];
uniform int 			iLightingModel;					// A value to control the specular lighting model
uniform float 			fSurfaceSmoothness;
uniform bool			bDrawNormals;

// --------------------------------------------------
// varying variables
// --------------------------------------------------
varying vec3 			V;								// Vertex in View-Space
varying vec3 			N;								// Normal of the Vertex
varying vec3 			vLightDir;						// Vec To Light

// --------------------------------------------------
// Vertex-Shader
// --------------------------------------------------
void main()
{
		// This will be the resulting position:
		vec4 vFinalVertex = vec4( 0.0, 0.0, 0.0, 0.0 );
		vec3 vFinalNormal = vec3( 0.0, 0.0, 0.0 );

		// Those two are used for shuffling
		vec4 vCurIndex = vIndex;
		vec4 vCurWeight = vWeight;

		// run through all assigned bones:
		for (int i=0; i< int( fNumBones ); ++i)
		{
				mat4 m44 = MatBones[ int( vCurIndex.x ) ];
				
				// transform the offset by bone i
				vFinalVertex += m44 * gl_Vertex * vCurWeight.x;

				mat3 m33 = mat3(m44[0].xyz,
								m44[1].xyz,
								m44[2].xyz);

				// transform normal by bone i
				vFinalNormal += m33 * gl_Normal * vCurWeight.x;

				// shift over the index/weight variables, this moves the index and 
				// weight for the current bone into the .x component of the index 
				// and weight variables
				vCurIndex = vCurIndex.yzwx;
				vCurWeight = vCurWeight.yzwx;
		}

		
		N = gl_NormalMatrix * vFinalNormal;
		V = ( gl_ModelViewMatrix * vFinalVertex ).xyz;
		vLightDir = gl_LightSource[0].position.xyz - V;
		
		gl_Position = gl_ModelViewProjectionMatrix * vFinalVertex;
}
